/***************************************************************************
 *   Copyright (C) 2007 by David Sansome <davidsansome@gmail.com>          *
 *                         Andrea Grandi <a.grandi@gmail.com>              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "ui_mainwindow.h"
#include "ui_createalbumdialog.h"
#include "album.h"
#include "service.h"

#include <QWidget>
#include <QPair>

class QKeyEvent;
class QSettings;
class QDialog;

class Photo
{
public:
	Photo(const QImage& i, const QString& f) : image(i), fileName(f), id(0) {}
	
	QImage image;
	QString fileName;
	qulonglong id;
};

class MainWindow : public QWidget
{
	Q_OBJECT
public:
	MainWindow(QWidget* parent = 0, Qt::WindowFlags f = 0);
	~MainWindow();

private slots:
	void addFacebook();
	void addFlickr();
	void addPicasa();
	void removeAccount();
	void accountSelected();
	
	void addPhoto();
	void removePhoto();
	void photoSelected();
	
	void createAlbum();
	void albumSelected();
	
	void uploadProgress(int bytes, int totalBytes);
	
	void morePhotosButtonClicked();
	void tagPhotos();
	void tagPhoto(QListWidgetItem* item);

private:
	void addService(Service* service, const QPixmap& pixmap);
	void loadAccounts();
	void saveAccounts();
	void setProgress(QWidget* abortPage, const QString& line1, const QString& line2 = QString::null, int maximum = 0);
	
	bool eventFilter(QObject* obj, QEvent* event);
	
	Ui_MainWindow m_ui;
	
	QSettings* m_settings;
	QList<Service*> m_services;
	
	QList<Photo> m_photos;
	QList<Album*> m_albums;
	
	Service* m_service;
	Album* m_album;
	
	QDialog* m_createAlbumDialog;
	Ui_CreateAlbumDialog m_createAlbumDialogUi;
	
	int m_abortPage;
};

#endif

